import { __generator, __read, __spreadArray } from "tslib";
import { createBrowserHistory } from "history";
import { applyMiddleware, compose, legacy_createStore as createStore } from "redux";
import { createReduxHistoryContext } from "redux-first-history";
import createSagaMiddleware, {} from "redux-saga";
import { call as rawCall, race as rawRace, take, takeEvery } from "redux-saga/effects";
import { LoggerImpl } from "./Logger";
import { executeAction } from "./module";
import { LOADING_ACTION, rootReducer } from "./reducer";
import { captureError } from "./util/error-util";
export var app = createApp();
export var logger = app.logger;
function composeWithDevTools(enhancer) {
    var composeEnhancers = compose;
    if (process.env.NODE_ENV === "development") {
        var extension = window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__;
        if (extension) {
            composeEnhancers = extension({
                // Ref: https://github.com/reduxjs/redux-devtools/blob/main/extension/docs/API/Arguments.md#actionsdenylist--actionsallowlist
                actionsDenylist: [LOADING_ACTION],
            });
        }
    }
    return composeEnhancers(enhancer);
}
function createApp() {
    var _a = createReduxHistoryContext({
        history: createBrowserHistory(),
    }), createReduxHistory = _a.createReduxHistory, routerMiddleware = _a.routerMiddleware, routerReducer = _a.routerReducer;
    var sagaMiddleware = createSagaMiddleware({
        onError: function (error, info) { return captureError(error, "@@framework/detached-saga", { extraStacktrace: info.sagaStack }); },
    });
    var store = createStore(rootReducer(routerReducer), composeWithDevTools(applyMiddleware(routerMiddleware, sagaMiddleware)));
    // createReduxHistory() will dispatch an action, it must be called before middleware takeEvery(*) takes effect
    var reduxHistory = createReduxHistory(store);
    sagaMiddleware.run(function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, takeEvery("*", function (action) {
                        var actionHandler, handler, moduleName;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    actionHandler = app.actionHandlers[action.type];
                                    if (!actionHandler) return [3 /*break*/, 2];
                                    handler = actionHandler.handler, moduleName = actionHandler.moduleName;
                                    // Cancel all saga when related module destroy
                                    // @see https://stackoverflow.com/a/45806187
                                    return [4 /*yield*/, rawRace({
                                            task: rawCall.apply(void 0, __spreadArray([executeAction, action.type, handler], __read(action.payload), false)),
                                            cancel: take("@@".concat(moduleName, "/@@cancel-saga")),
                                        })];
                                case 1:
                                    // Cancel all saga when related module destroy
                                    // @see https://stackoverflow.com/a/45806187
                                    _a.sent();
                                    _a.label = 2;
                                case 2: return [2 /*return*/];
                            }
                        });
                    })];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
    return {
        history: reduxHistory,
        store: store,
        sagaMiddleware: sagaMiddleware,
        actionHandlers: {},
        logger: new LoggerImpl(),
        loggerConfig: null,
        errorHandler: function () { return __generator(this, function (_a) {
            return [2 /*return*/];
        }); },
    };
}
//# sourceMappingURL=app.js.map