import { app } from "../../app";
var StartupModulePerformanceLogger = /** @class */ (function () {
    function StartupModulePerformanceLogger() {
    }
    StartupModulePerformanceLogger.registerIfNotExist = function (moduleName) {
        if (!StartupModulePerformanceLogger.moduleName) {
            StartupModulePerformanceLogger.moduleName = moduleName;
        }
    };
    StartupModulePerformanceLogger.log = function (currentModuleName) {
        var _a;
        if (currentModuleName === StartupModulePerformanceLogger.moduleName) {
            var actionName = "".concat(currentModuleName, "/@@STARTUP_PERF");
            var stats = {};
            if (window.performance && typeof performance.getEntriesByType === "function") {
                var entry = performance.getEntriesByType("navigation")[0];
                if (entry) {
                    // ref: https://developer.mozilla.org/en-US/docs/Web/API/PerformanceNavigationTiming
                    stats["http_start"] = Math.round(entry.requestStart);
                    stats["http_end"] = Math.round(entry.responseEnd);
                    stats["dom_start"] = Math.round(entry.domInteractive);
                    stats["dom_content"] = Math.round(entry.domContentLoadedEventEnd);
                    stats["dom_end"] = Math.round(entry.loadEventEnd);
                }
            }
            var totalDuration = Date.now() - StartupModulePerformanceLogger.startupTime;
            var slowStartupThreshold = ((_a = app.loggerConfig) === null || _a === void 0 ? void 0 : _a.slowStartupThresholdInSecond) || 5;
            if (totalDuration / 1000 >= slowStartupThreshold) {
                app.logger.warn({
                    action: actionName,
                    elapsedTime: totalDuration,
                    stats: stats,
                    errorCode: "SLOW_STARTUP",
                    errorMessage: "Startup took ".concat((totalDuration / 1000).toFixed(2), " sec, longer than ").concat(slowStartupThreshold),
                });
            }
            else {
                app.logger.info({
                    action: actionName,
                    elapsedTime: totalDuration,
                    stats: stats,
                });
            }
        }
    };
    StartupModulePerformanceLogger.startupTime = Date.now();
    return StartupModulePerformanceLogger;
}());
export { StartupModulePerformanceLogger };
//# sourceMappingURL=StartupModulePerformanceLogger.js.map