import type { Method } from "axios";
import { APIException, NetworkConnectionException } from "../Exception";
export interface SSEConfig<Request> {
    actionPrefix: string;
    url: string;
    method?: Method;
    payload?: Request;
    logResponse?: boolean;
}
type Unsubscriber = () => void;
type ConnectedListener = (connectedTimes: number) => void;
type ErrorListener = (error: NetworkConnectionException | APIException) => void;
export interface SSE<Response extends Record<string, any>> {
    connect: () => Promise<void>;
    disconnect: () => void;
    onConnected: (listener: ConnectedListener) => Unsubscriber;
    onError: (listener: ErrorListener) => Unsubscriber;
    onResponse: <E extends keyof Response>(event: E, listener: (data: NonNullable<Response[E]>) => void) => Unsubscriber;
}
export declare function sse<Request, Response extends Record<string, any>>({ actionPrefix, url, method, payload, logResponse, }: SSEConfig<Request>): SSE<Response>;
export {};
//# sourceMappingURL=sse.d.ts.map