import { __assign, __awaiter, __generator } from "tslib";
import { ErrorEvent, EventSource } from "eventsource";
import { parseWithDate } from "./json-util";
import { uuid } from "./uuid";
import { app } from "../app";
import { APIException, NetworkConnectionException } from "../Exception";
export function sse(_a) {
    var _this = this;
    var 
    // prettier
    actionPrefix = _a.actionPrefix, url = _a.url, _b = _a.method, method = _b === void 0 ? "GET" : _b, payload = _a.payload, _c = _a.logResponse, logResponse = _c === void 0 ? false : _c;
    var eventSource = null;
    var startTime = null;
    var connectingTimes = 1;
    var traceId = null;
    var connectedListeners = [];
    var errorListeners = [];
    var messageListeners = new Map();
    var errorEventToNetworkException = function (e) { return new NetworkConnectionException("Failed to connect SSE: ".concat(url), url, "".concat(e.type || "UNKNOWN", ": ").concat(e.message || "UNKNOWN")); };
    var generalMessageListener = function (e) {
        var data = safeParse(e.data);
        if (!data) {
            if (logResponse) {
                app.logger.info({
                    action: "".concat(actionPrefix, "/@@SSE_RESPONSE"),
                    context: { sse_url: url, trace_id: traceId || undefined },
                    info: { message: e.data },
                });
            }
            return;
        }
        var validMessageFields = Object.keys(data).filter(function (_) { return data[_] !== null; });
        if (validMessageFields.length > 0) {
            validMessageFields.forEach(function (field) { var _a; return (_a = messageListeners.get(field)) === null || _a === void 0 ? void 0 : _a.forEach(function (listener) { return listener(data[field]); }); });
            if (logResponse) {
                app.logger.info({
                    action: "".concat(actionPrefix, "/@@SSE_RESPONSE"),
                    context: { sse_url: url, trace_id: traceId || undefined },
                    info: { message: validMessageFields.map(function (field) { return "".concat(field.toString(), ":").concat(JSON.stringify(data[field])); }).join("\n") },
                });
            }
        }
    };
    var generalErrorListener = function (e) {
        var exception;
        if (e.type === "error" && e instanceof MessageEvent && e.data) {
            // a special case for backend API exception, which will be followed by an error event (triggered by server-side close)
            var errorResponse = safeParse(e.data);
            var message = (errorResponse === null || errorResponse === void 0 ? void 0 : errorResponse.message) || "[No Response]";
            exception = new APIException(message, 0, url, e.data, (errorResponse === null || errorResponse === void 0 ? void 0 : errorResponse.id) || null, (errorResponse === null || errorResponse === void 0 ? void 0 : errorResponse.errorCode) || null);
        }
        else {
            exception = errorEventToNetworkException(e);
        }
        app.logger.exception(exception, {
            action: "".concat(actionPrefix, "/@@SSE_ERROR"),
            context: { sse_url: url, trace_id: traceId || undefined },
        });
        errorListeners.forEach(function (listener) { return listener(exception); });
        startTime = Date.now(); // reset startTime for next connection
    };
    return {
        connect: function () { return __awaiter(_this, void 0, void 0, function () {
            var sseState;
            return __generator(this, function (_a) {
                sseState = eventSource === null || eventSource === void 0 ? void 0 : eventSource.readyState;
                if (sseState === EventSource.CONNECTING || sseState === EventSource.OPEN)
                    return [2 /*return*/];
                startTime = Date.now();
                traceId = uuid();
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        eventSource = new EventSource(url, {
                            fetch: function (input, init) {
                                return fetch(input, __assign(__assign({}, init), { method: method, body: payload ? JSON.stringify(payload) : null, headers: __assign(__assign({}, init === null || init === void 0 ? void 0 : init.headers), { "x-trace-id": traceId, "content-type": "application/json" }) }));
                            },
                        });
                        eventSource.onopen = function (e) {
                            app.logger.info({
                                action: "".concat(actionPrefix, "/@@SSE_CONNECTED"),
                                context: { sse_url: url, trace_id: traceId || undefined },
                                stats: { connecting_times: connectingTimes },
                                elapsedTime: startTime ? Date.now() - startTime : undefined,
                            });
                            // onopen callback may be called many times by EventSource auto-retry
                            startTime = null;
                            connectingTimes++;
                            eventSource.onerror = null;
                            eventSource.addEventListener("message", generalMessageListener);
                            eventSource.addEventListener("error", generalErrorListener);
                            connectedListeners.forEach(function (listener) { return listener(connectingTimes); });
                            resolve();
                        };
                        // for create connection error
                        eventSource.onerror = function (e) {
                            eventSource.onopen = null;
                            eventSource.onerror = null;
                            eventSource.close();
                            var exception = errorEventToNetworkException(e);
                            app.logger.exception(exception, {
                                action: "".concat(actionPrefix, "/@@SSE_CONNECT_ERROR"),
                                context: { sse_url: url, trace_id: traceId || undefined },
                                elapsedTime: startTime ? Date.now() - startTime : undefined,
                            });
                            reject(exception);
                        };
                    })];
            });
        }); },
        disconnect: function () {
            connectedListeners.splice(0, connectedListeners.length);
            errorListeners.splice(0, errorListeners.length);
            messageListeners.clear();
            if (eventSource) {
                eventSource.onopen = null;
                eventSource.removeEventListener("message", generalMessageListener);
                eventSource.removeEventListener("error", generalErrorListener);
                eventSource.close();
                eventSource = null;
            }
        },
        onConnected: function (listener) {
            connectedListeners.push(listener);
            return function () {
                var index = connectedListeners.indexOf(listener);
                if (index !== -1)
                    connectedListeners.splice(index, 1);
            };
        },
        onError: function (listener) {
            errorListeners.push(listener);
            return function () {
                var index = errorListeners.indexOf(listener);
                if (index !== -1)
                    errorListeners.splice(index, 1);
            };
        },
        onResponse: function (event, listener) {
            var listeners = messageListeners.get(event);
            if (!listeners) {
                messageListeners.set(event, [listener]);
            }
            else {
                listeners.push(listener);
            }
            return function () {
                var listeners = messageListeners.get(event);
                if (!listeners)
                    return;
                var index = listeners.indexOf(listener);
                if (index !== -1)
                    listeners.splice(index, 1);
            };
        },
    };
}
function safeParse(data) {
    try {
        return parseWithDate(data);
    }
    catch (e) {
        return null;
    }
}
//# sourceMappingURL=sse.js.map